require_relative '../../../../spec/spec_helper'

describe 'Alter table parsing' do
  let(:parser) { DDLParser::DDL::DB2::Parser.new }
  context 'alter table' do
    it 'parses alter_table add column' do
      sql = <<EOF
            ALTER TABLE BUDGET
              ADD COLUMN BUDGET_AMOUNT_IN_CO DECIMAL(15,2) NOT NULL DEFAULT 0
              ADD COLUMN BUDGET_AMOUNT_CURR CHARACTER(4)
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect
      result.count.should == 3
      result.should include(:operation, :table_name)
      result.should include(:operation => 'alter table')
      result[:elements].first[:add][:column].should include(:field, :data_type)
    end

    it 'parses alter_table foreign key constraint without on' do
      sql = <<EOF
ALTER TABLE REPLAN_ADDR
        ADD CONSTRAINT REPLAN_ADDR_FK
            FOREIGN KEY
                (CONSIGNMENT,
                EVENT)
            REFERENCES REPLAN_MAIN
                (CONSIGNMENT,
                EVENT)
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect #.count.should == 3
      result.should include({:operation => 'alter table', :table_name=> 'replan_addr'})
      result[:elements].first[:add_constraint].should include({:constraint_name=> 'replan_addr_fk'})
    end

    it 'parses foreign key clause' do
      sql = <<EOF
FOREIGN KEY
                (CONSIGNMENT,
                EVENT)
            REFERENCES REPLAN_MAIN
                (CONSIGNMENT,
                EVENT)
            ON DELETE RESTRICT
            ON UPDATE NO ACTION
EOF
      begin
        result = parser.foreign_key_clause.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect
      result.count.should == 2
      result.should include(:reference_clause)
      result[:member_fields][0][:field].should == 'consignment'
      result[:member_fields][1][:field].should == 'event'
      result[:reference_clause][:reference_arglist][0][:item].should == 'consignment'
      result[:reference_clause][:reference_arglist][1][:item].should == 'event'
    end
    it 'parses reference_clause' do
      sql = <<EOF
REFERENCES REPLAN_MAIN
                (CONSIGNMENT,
                EVENT)

EOF
      begin
        result = parser.reference_clause.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect
      result.should include(:reference_arglist)
      result[:reference_arglist][0][:item].should == 'consignment'
      result[:reference_arglist][1][:item].should == 'event'
    end
    it 'parses on_delete_clause' do
      sql = <<EOF
ON DELETE RESTRICT
EOF
      begin
        result = parser.on_delete_clause.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect
      result.should include(:on_delete_option => 'restrict')
    end

    it 'parses on_update_clause' do
      sql = <<EOF
ON UPDATE NO ACTION
EOF
      begin
        result = parser.on_update_clause.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect
      result.should include(:on_update_option => 'no action')
    end


    it 'parses complete alter_table' do
      sql = <<EOF
ALTER TABLE REPLAN_ADDR
        ADD CONSTRAINT REPLAN_ADDR_FK
            FOREIGN KEY
                (CONSIGNMENT,
                EVENT)
            REFERENCES REPLAN_MAIN
                (CONSIGNMENT,
                EVENT)
            ON DELETE RESTRICT
            ON UPDATE RESTRICT
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      #puts result.inspect
      result.should include(:operation, :table_name)
      result.should include(:operation => 'alter table')
      result.should include(:table_name => 'replan_addr')
      result[:elements].first.should include(:add_constraint)
      result[:elements].first[:add_constraint].should include(:constraint_name => 'replan_addr_fk')
      result[:elements].first[:add_constraint].should include(:foreign_key_clause)
      result[:elements].first[:add_constraint][:foreign_key_clause].should include(:member_fields)
      result[:elements].first[:add_constraint][:foreign_key_clause][:member_fields][0].should include(:field=>'consignment')
      result[:elements].first[:add_constraint][:foreign_key_clause][:member_fields][1].should include(:field=>'event')
      result[:elements].first[:add_constraint][:foreign_key_clause].should include(:reference_clause)
      result[:elements].first[:add_constraint][:foreign_key_clause][:reference_clause].should include(:reference_arglist)
      result[:elements].first[:add_constraint][:foreign_key_clause][:reference_clause][:reference_arglist][0].should include(:item=>'consignment')
      result[:elements].first[:add_constraint][:foreign_key_clause][:reference_clause][:reference_arglist][1].should include(:item=>'event')

    end

  end
end