require_relative '../../../spec/spec_helper'

class DummyParser < Parslet::Parser
  include DDLParser::SharedRules::Constants
end

describe DDLParser::SharedRules::Constants do
  let(:parser) { DummyParser.new }

  it 'digits' do
    (0..9).each do |digit|
      expect(parser.digit).to     parse(digit.to_s)
    end
    ('a'..'z').each do |non_digit|
      expect(parser.digit).not_to     parse(non_digit.to_s)
    end
  end

  it 'integer' do
    expect(parser.integer).to     parse('1')
    expect(parser.integer).to     parse('123456789')
    expect(parser.integer).to     parse('-123456789')
    expect(parser.integer).to     parse('0123')
    expect(parser.integer).not_to parse('1.2')
  end

  it 'float' do
    expect(parser.float).to     parse('-1.1')
    expect(parser.float).to     parse('1.1')
    expect(parser.float).to     parse('123456.789')
    expect(parser.float).to     parse('-123456.789')
    expect(parser.float).not_to parse('12')
  end

  it 'space' do
    expect(parser.space).to     parse(' ')
    expect(parser.space).to     parse('  ')
    expect(parser.space).to     parse("\t")
    expect(parser.space).not_to parse('')
    expect(parser.space).not_to parse('1')
  end

  it 'space?' do
    expect(parser.space?).to     parse('')
    expect(parser.space?).to     parse(' ')
    expect(parser.space?).to     parse('  ')
    expect(parser.space?).not_to parse('1')
  end

  it 'newline' do
    expect(parser.newline).to     parse("\n")
    expect(parser.newline).not_to parse(' ')
    expect(parser.newline).not_to parse("\t")
  end

  it 'spaces' do
    expect(parser.spaces).to     parse("\n")
    expect(parser.spaces).to     parse(" \n \n")
    expect(parser.spaces).to     parse("\t ")
  end

  it 'comment?' do
    expect(parser.comment?).to     parse('-- foobar')
    expect(parser.comment?).not_to parse("--\nfoobar")
  end

  it 'comma' do
    expect(parser.comma).to     parse(',')
    expect(parser.comma).to     parse(', ')
    expect(parser.comma).not_to parse(' , ')
  end

  it 'lparen' do
    expect(parser.lparen).to     parse('(')
    expect(parser.lparen).to     parse('( ')
    expect(parser.lparen).not_to parse(' (')
  end

  it 'rparen' do
    expect(parser.rparen).to     parse(')')
    expect(parser.rparen).to     parse(') ')
    expect(parser.rparen).not_to parse(' )')
  end

  it 'boolean' do
    expect(parser.boolean).to     parse('true')
    expect(parser.boolean).to     parse('false')
    expect(parser.boolean).not_to parse('false ')
  end

  it 'datetime' do
    expect(parser.datetime).to    parse('2014-02-28T12:01:02Z')
  end

  it 'string' do
    expect(parser.string).to      parse("'foobar'")
    expect(parser.string).to      parse("'foo bar'")
    expect(parser.string).not_to  parse("'foo bar")
    expect(parser.string).not_to  parse('"foobar"')
  end

  it 'const' do
    expect(parser.const).to      parse("'foobar'")
    expect(parser.const).to      parse('1')
    expect(parser.const).to      parse('1.1')
    expect(parser.const).to      parse('true')
    expect(parser.const).to      parse('2014-02-28T12:01:02Z')
    expect(parser.const).not_to  parse('(')
    expect(parser.const).not_to  parse(',')
  end

  it 'identifier' do
    expect(parser.identifier).to      parse('foobar')
    expect(parser.identifier).not_to  parse('foo bar')
    expect(parser.identifier).to     parse('a')
    expect(parser.identifier).to     parse('aB')
    expect(parser.identifier).to     parse('A1')
    expect(parser.identifier).to     parse('A1_2')
    expect(parser.identifier).not_to parse('A B')
  end

  it 'current_timestamp' do
    expect(parser.current_timestamp).to      parse('current timestamp')
  end

end
